/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */
package org.objectweb.clif.isac.plugins.httpmatrix10.actions;

import java.io.IOException;
import java.util.Map;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.objectweb.clif.isac.plugins.httpmatrix10.SessionObject;
import org.objectweb.clif.scenario.transitions.Transition;
import org.objectweb.clif.scenario.transitions.TransitionTable;
import org.objectweb.clif.storage.api.ActionEvent;

/**
 * This class implements the matrix sample methods
 *
 * @author JC Meillaud
 * @author A Peyrard
 * @author Bruno Dillenseger
 */
public class MatrixSamples {
	// definition of constants
	private static final String METHOD = "method";
	private static final String POST_METHOD = "POST";
	private static final String GET_METHOD = "GET";

	/**
	 * Sample do next, execute the next url
	 *
	 * @param sessionObject
	 *            The sessionObject
	 */
	public static ActionEvent doNext(SessionObject sessionObject, Map params,
			ActionEvent report) {
		// get the method value
		String methodType = (String)params.get(METHOD);
		// Set current date
		report.setDate(System.currentTimeMillis());
		// get the transition table
		TransitionTable transitionTable = sessionObject.getTransitionTable();
		// get the next transition
		Transition transition = transitionTable.getNextTransition();

		// if the table is finished
		if (transition.isEndSession()) {
			report.successful = true;
			report.comment = "endSession";
			report.duration = 0;
			// set the finish state in the session object
			sessionObject.setFinishState(true);
			return report;
		}
		// get the actions for this state
		String[] actions = transition.getAction();
		// get the url
		String url = actions[0];
		// create the http method
		HttpMethod method = null;
		if (methodType.equals(GET_METHOD)) {
			method = new GetMethod(url);
		}
		else if (methodType.equals(POST_METHOD)) {
			method = new PostMethod(url);
		}
		else {
			report.successful = false;
			report.comment = "Unknow http injection method : "+methodType;
			report.duration = 0;
			return report;
		}
		method.setFollowRedirects(true);
		method.getParams().makeLenient();
		// Set report comment
		report.comment = url;
		// exceute the method
		try {
			sessionObject.getHttpClient().executeMethod(method);
			// get response status
			report.result = method.getStatusCode() + " - "
					+ method.getStatusText();
			report.successful = true;
		} catch (IOException ioex) {
			report.comment = "Unable to execute http method on " + url + " ->"
					+ ioex;
			report.successful = false;
		} finally {
			// always release the connection after we're done
			method.releaseConnection();
		}

		// update the report datas
		report.duration = (int) (System.currentTimeMillis() - report.getDate());

		// return the report datas
		return report;
	}
}