/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2006 F.Barreca & I.Vassilev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF
 *
 * Contact: clif@objectweb.org
 */

package org.objectweb.clif.isac.plugins.sipinjector10;

/**
 * @author Fabien Barreca
 * @author Iassen Vassilev
 */
public class Semaphore {
	private int value;
	private int recordedValue;
	private long timeOut;
	
	/**
	 * Constructor
	 * 
	 * @param value : the default value of the semaphore
	 * @param timeout : the timeout wich will be used for every P 
	 */
	public Semaphore(int value, long timeout)
	{
		this.value=value;
		this.recordedValue=value;
		this.timeOut=timeout;
	}
	
	public int getValeur()
	{
		int result=recordedValue;
		recordedValue=0;
		return result;
	}
	
	/**
	 * Classic semaphore "P" action. Blocks the caller until a "V" is called on him
	 * 
	 */
	synchronized public void P()
	{
		while (value==0)
		{
			try 
			{
				wait(this.timeOut);
				
				//if the Thread wakes up because of a timeout
				if (value==0)
				{
					return;
				}
			}
			catch(InterruptedException e)
			{
				System.out.println("Erreur de S�maphore");
			}
		}
		value=0;
	}
	
	/**
	 * Classic semaphore "V" action. Frees the object on wich it is called
	 * sets a response code in the "mailbox"
	 * 
	 * @param
	 * 		lastResponseCode : parameter to be send to the mailbox of the object 
	 * 		when freeing it of the semaphore
	 * 
	 */
	synchronized public void V(int lastResponseCode)
	{
		value=lastResponseCode;
		recordedValue=lastResponseCode;
		notify();
	}

}
