package org.objectweb.isac.plugin.udpinjector;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.net.SocketException;
import java.util.Map;
import java.util.HashMap;
import java.util.Hashtable;
import org.objectweb.clif.scenario.isac.plugin.SessionObjectAction;
import org.objectweb.clif.scenario.isac.plugin.DataProvider;
import org.objectweb.clif.scenario.isac.exception.IsacRuntimeException;
import org.objectweb.clif.scenario.isac.plugin.ControlAction;
import org.objectweb.clif.scenario.isac.plugin.SampleAction;
import org.objectweb.clif.scenario.isac.plugin.TestAction;
import org.objectweb.clif.storage.api.ActionEvent;

/**
 * Implementation of a session object for plugin ~UDPinjector~
 */
public class SessionObject
	extends DatagramSocket
	implements SessionObjectAction, DataProvider, ControlAction, SampleAction, TestAction
{
	static final int TEST_ISCONNECTED = 4;
	static final int TEST_ISNOTCONNECTED = 5;
	static final int TEST_ISBOUND = 6;
	static final int TEST_ISNOTBOUND = 7;
	static final int TEST_ISCLOSED = 11;
	static final int TEST_ISNOTCLOSED = 12;
	static final int SAMPLE_CONNECT = 2;
	static final String SAMPLE_CONNECT_PORT = "port";
	static final String SAMPLE_CONNECT_HOST = "host";
	static final int SAMPLE_SEND = 13;
	static final String SAMPLE_SEND_DATA = "data";
	static final int SAMPLE_RECEIVE = 14;
	static final String SAMPLE_RECEIVE_VARIABLE = "variable";
	static final int CONTROL_BIND = 0;
	static final String CONTROL_BIND_PORT = "port";
	static final String CONTROL_BIND_HOST = "host";
	static final int CONTROL_CLOSE = 1;
	static final int CONTROL_DISCONNECT = 3;
	static final int CONTROL_SETRCVBUFSIZE = 8;
	static final String CONTROL_SETRCVBUFSIZE_SIZE = "size";
	static final int CONTROL_SETSNDBUFSIZE = 9;
	static final String CONTROL_SETSNDBUFSIZE_SIZE = "size";
	static final int CONTROL_SETTIMEOUT = 10;
	static final String CONTROL_SETTIMEOUT_TIMEOUT = "timeout";

	static final String SEND_TYPE = "SEND";
	static final String RECEIVE_TYPE = "RECEIVE";
	static final String CONNECT_TYPE = "CONNECT";

	private Map<String,String> buffers;

	/**
	 * Constructor for specimen object.
	 *
	 * @param params key-value pairs for plugin parameters
	 */
	public SessionObject(Hashtable params) throws SocketException
	{
		super((SocketAddress)null);
	}

	/**
	 * Copy constructor (clone specimen object to get session object).
	 *
	 * @param so specimen object to clone
	 */
	private SessionObject(SessionObject so) throws SocketException
	{
		super((SocketAddress)null);
		buffers = new HashMap<String,String>();
	}


	////////////////////////////////////////
	// SessionObjectAction implementation //
	////////////////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() throws IsacRuntimeException
	{
		try
		{
			return new SessionObject(this);
		}
		catch (SocketException ex)
		{
			throw new IsacRuntimeException("Can't create UDPinjector session object", ex);
		}
	}

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close()
	{
		super.close();
	}

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset()
	{
		buffers.clear();
		super.disconnect();
	}


	/////////////////////////////////
	// DataProvider implementation //
	/////////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.DataProvider#doGet()
	 */
	public String doGet(String var)
	{
		return buffers.get(var);
	}

	
	//////////////////////////////////
	// ControlAction implementation //
	//////////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.ControlAction#doControl()
	 */
	public void doControl(int number, Map params)
	{
		switch (number)
		{
			case CONTROL_SETTIMEOUT:
				try
				{
					setSoTimeout(Integer.parseInt((String)params.get(CONTROL_SETTIMEOUT_TIMEOUT)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket timeout", ex);
				}
			case CONTROL_SETSNDBUFSIZE:
				try
				{
					setSendBufferSize(Integer.parseInt((String)params.get(CONTROL_SETSNDBUFSIZE_SIZE)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket send buffer size", ex);
				}
			case CONTROL_SETRCVBUFSIZE:
				try
				{
					setReceiveBufferSize(Integer.parseInt((String)params.get(CONTROL_SETRCVBUFSIZE_SIZE)));
					break;
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't set UDPinjector socket receive buffer size", ex);
				}
			case CONTROL_DISCONNECT:
				disconnect();
				break;
			case CONTROL_CLOSE:
				close();
				break;
			case CONTROL_BIND:
				try
				{
					bind(
						new InetSocketAddress(
							(String)params.get(CONTROL_BIND_HOST),
							Integer.parseInt((String)params.get(CONTROL_BIND_PORT))));
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't bind UDPinjector socket", ex);
				}
				break;
			default:
				throw new Error("Unable to find this control in ~UDPinjector~ ISAC plugin: " + number);
		}
	}


	/////////////////////////////////
	// SampleAction implementation //
	/////////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.SampleAction#doSample()
	 */
	public ActionEvent doSample(int number, Map params, ActionEvent report)
	{
		switch (number)
		{
			case SAMPLE_RECEIVE:
				if (! isBound())
				{
					throw new IsacRuntimeException("UDPinjector must be bound before receiving data.");
				}
				String variable = (String)params.get(SAMPLE_RECEIVE_VARIABLE);
				report.type = RECEIVE_TYPE;
				byte[] buffer;
				try
				{
					buffer = new byte[getReceiveBufferSize()];
				}
				catch (SocketException ex)
				{
					throw new IsacRuntimeException("Can't get size of socket receive buffer in UDPinjector.", ex);
				}
				DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
				report.setDate(System.currentTimeMillis());
				try
				{
					receive(packet);
					report.duration = (int)(System.currentTimeMillis() - report.getDate());	
					report.successful = true;
					report.comment = String.valueOf(packet.getLength()) + " bytes received";
					if (variable != null && !variable.equals(""))
					{
						buffers.put(variable, new String(buffer));
					}
				}
				catch (Exception ex)
				{
					report.successful = false;
					report.result = ex.toString();
				}
				return report;
			case SAMPLE_SEND:
				if (! isConnected())
				{
					throw new IsacRuntimeException("UDPinjector must be connected before sending data.");
				}
				byte[] data = ((String)params.get(SAMPLE_SEND_DATA)).getBytes();
				report.type = SEND_TYPE;
				report.comment = String.valueOf(data.length) + " bytes";
				report.setDate(System.currentTimeMillis());
				try
				{
					send(new DatagramPacket(data, data.length));
					report.duration = (int)(System.currentTimeMillis() - report.getDate());	
					report.successful = true;
				}
				catch (Exception ex)
				{
					report.successful = false;
					report.result = ex.toString();
				}
				return report;
			case SAMPLE_CONNECT:
				String host = (String)params.get(SAMPLE_CONNECT_HOST);
				int port = Integer.parseInt((String)params.get(SAMPLE_CONNECT_PORT));
				report.type = CONNECT_TYPE;
				report.setDate(System.currentTimeMillis());
				try
				{
					connect(new InetSocketAddress(host, port));
					report.duration = (int)(System.currentTimeMillis() - report.getDate());
					report.successful = true;
					report.result = "connected to " + host + ":" + port;
				}
				catch (SocketException ex)
				{
					report.successful = false;
					report.comment = "Can't connect UDPinjector socket to " + host + ":" + port;
					report.result = ex.toString();
				}
				return report;
			default:
				throw new Error("Unable to find this sample in ~UDPinjector~ ISAC plugin: " + number);
		}
	}


	///////////////////////////////
	// TestAction implementation //
	///////////////////////////////

	/**
	 * @see org.objectweb.clif.scenario.isac.plugin.TestAction#doTest()
	 */
	public boolean doTest(int number, Map params)
	{
		switch (number)
		{
			case TEST_ISNOTCLOSED:
				return ! isClosed();
			case TEST_ISCLOSED:
				return isClosed();
			case TEST_ISNOTBOUND:
				return ! isBound();
			case TEST_ISBOUND:
				return isBound();
			case TEST_ISNOTCONNECTED:
				return ! isConnected();
			case TEST_ISCONNECTED:
				return isConnected();
			default:
				throw new Error("Unable to find this test in ~UDPinjector~ ISAC plugin: " + number);
		}
	}
}
